--[[ ButtonFrame ]]

TEB_ButtonFrame = {}


function TEB_ButtonFrame_New(id, parent)
	local bf = _G[parent:GetName().."ButtonFrame"..id]
	if not(bf) then
		bf = CreateFrame("Frame", parent:GetName().."ButtonFrame"..id, UIParent, "TinyExtraBarsButtonFrameTemplate")
	end
	bf.container = parent
	bf:SetID(id)

	-- settings
	bf.title = TinyExtraBarsPC:Get({'Containers', parent:GetID(), 'tabs', id, "title"}, 'Tab'..id)
	bf.visibility = TinyExtraBarsPC:Get({'Containers', parent:GetID(), 'tabs', id, "Visibility"}, nil)
	if not(bf.visibility) then
		bf.visibility = {
			["Talents"] = {
				[1] = true,
				[2] = true,
			},
			["Stance"] = {
				[1] = true,
				[2] = false,
				[3] = false,
				[4] = false,
				[5] = false,
				[6] = false,
				[7] = false,
				[8] = false,
				[9] = false,
			},
			["Vehicle"] = { 
				[1] = false,
				[2] = true,
			},
			["BonusBar"] = {
				[1] = false,
				[2] = true,
			},
			["Custom"] = "",
		}
	end
	
	-- adding methods to frame
	for k, v in pairs(TEB_ButtonFrame) do
		if type(v) == "function" then
			bf[k] = v
		end
	end
	
	bf.ButtonList = {}
	
	bf:SetAttribute("_onstate-combat", [[self:ChildUpdate('combat', newstate)]]) --print("bf on combat"); 
	--RegisterStateDriver(bf, "combat", "[nocombat] 0; [combat] 1;")

	bf:UpdateVisibilityDriver()
	
	bf:RegisterEvents()
	
	return bf
end

function TEB_ButtonFrame:SaveVisibilityDriver()
	TinyExtraBarsPC:Set({'Containers', self.container:GetID(), 'tabs', self:GetID(), "Visibility"}, self.visibility)
end

function TEB_ButtonFrame:UpdateVisibilityDriver()
	if TEB_SettingsMode then
		self:DisableVisibilityDriver()
		return
	end

	self:SaveVisibilityDriver()
	
	local text = ""
	--if (TEB_SettingsMode) then
	--	text = text.."[nocombat] show; "
	--end
	if not(self.visibility["Talents"][1]) then
		text = text.."[spec:1] hide; "
	end
	if not(self.visibility["Talents"][2]) then
		text = text.."[spec:2] hide; "
	end
	if (self.visibility["Vehicle"][2]) then
		text = text.."[vehicleui] hide; "
	end
	if (self.visibility["BonusBar"][2]) then
		text = text.."[overridebar] hide; "
	end
	if not(self.visibility["Stance"][1]) then
		local temp = ""
		for i = 2, #self.visibility["Stance"] do
			if not(self.visibility["Stance"][i]) then
				temp = temp..(i - 2).."/"
			end
		end
		if temp:len() > 0 then
			temp = temp:sub(1, temp:len() - 1)
			text = text.."[stance:"..temp.."] hide; "
		end
	end
	
	--print(text)
	local customText = self.visibility["Custom"]
	if (customText ~= "") then
		RegisterStateDriver(self, "visibility", text..customText)
	else
		if (text ~= "") then
			RegisterStateDriver(self, "visibility", text.."show")
		else
			UnregisterStateDriver(self, "visibility")
		end
	end
end

function TEB_ButtonFrame:DisableVisibilityDriver()
	UnregisterStateDriver(self, "visibility")
end

function TEB_ButtonFrame:SetAnchor(parent)
	self:ClearAllPoints()
	self:SetPoint("TOPLEFT", parent, "TOPLEFT", 10, -10)
end

function TEB_ButtonFrame:SetSize(rows, cols)
	local width = TinyExtraBars_GetButtonsTotalSize(cols)
	local height = TinyExtraBars_GetButtonsTotalSize(rows)
	
	self:SetWidth(width)
	self:SetHeight(height)
end

function TEB_ButtonFrame:SetButtons(rows, cols)
	for r = 1, rows do
		for c = 1, cols do
			local btn = TEB_Button_AttachToFrame(self, r, c)
			local v = TinyExtraBarsPC:Get({'Containers', self.container:GetID(), 'tabs', self:GetID(), r, c}, nil)
			if (v) and (TEB_ACCEPTABLE_COMMANDS[v.command]) then
				if (v.command == "spell") then
					if not(v.value) then
						print(tostring(v.command).." "..tostring(v.subValue).." removed")
						btn:SaveCommand(nil, nil, nil, nil)
						btn:Set(nil, nil, nil, nil)
						btn:SetKey(nil)
					else
						if not(v.id) then
							--fix broken spells
							v.id = TinyExtraBars_FindSpellId(v.value)
							--print(tostring(v.value).." fixed id="..tostring(v.id))
						end
						btn:Set(v.command, v.value, v.subValue, v.id)
						btn:SetKey(v.keybind)
					end
				else
					if not(v.value) then
						print(tostring(v.command).." "..tostring(v.subValue).." removed")
						btn:SaveCommand(nil, nil, nil, nil)
						btn:Set(nil, nil, nil, nil)
						btn:SetKey(nil)
					else
						btn:Set(v.command, v.value, v.subValue, v.id)
						btn:SetKey(v.keybind)
					end
				end
			else
				btn:Set(nil, nil, nil, nil)
				btn:SetKey(nil)
			end
			if TEB_SettingsMode or btn.command then
				btn:Show()
			else
				btn:Hide()
			end
		end
	end
end

function TEB_ButtonFrame:SetButtonsCount(rows, cols)
	self:SetSize(rows, cols)
	
	-- hide unused buttons
	
	for r = 1, self.container.maxRows do
		for c = 1, self.container.maxCols do
			if r > rows or c > cols then
				local btn = self.ButtonList[r][c]
				if btn then
					--print("hiding "..r.." "..c)
					btn:HideButton()
				end
			end
		end
	end
end

function TEB_ButtonFrame:OnShow()
	--if ShowBorders then
	--	self:SetBackdrop({edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border", tile = true, tileSize = 8, edgeSize = 8, insets = { left = 4, right = 4, top = 4, bottom = 4 }})
	--else
	--	self:SetBackdrop({edgeFile = ""})
	--end
end

function TEB_ButtonFrame:OnHide()
end

local function EventShowGrid(self, ...)
	self:SetFrameStrata("DIALOG")
end

local function EventHideGrid(self, ...)
	self:SetFrameStrata(self.container.strata)
end

function TEB_ButtonFrame:OnEvent(event, ...)
	if self.EventHandlersTable[event] then
		self.EventHandlersTable[event](self, ...)
	end
end

function TEB_ButtonFrame:RegisterEvents()
	--event table
	self.EventHandlersTable = {
		["ACTIONBAR_SHOWGRID"] 			= EventShowGrid,
		["ACTIONBAR_HIDEGRID"] 			= EventHideGrid,
	}
	self:RegisterEvent("ACTIONBAR_SHOWGRID")
	self:RegisterEvent("ACTIONBAR_HIDEGRID")
	
	self:SetScript("OnEvent", TEB_ButtonFrame.OnEvent)	
end

function TEB_ButtonFrame:UnregisterEvents()
	if self.EventHandlersTable then
		for k, _ in pairs(self.EventHandlersTable) do
			self:UnregisterEvent(k)
		end
	end
	self.EventHandlersTable = {}
end
